#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.shared.view.interfaces import VIEWS_MERGERS_VOCABULARY
from pyams_portal.interfaces import IPortletSettings

# import packages
from pyams_content.shared.view import WfView
from pyams_content.shared.view.merge import CONCAT_VIEWS_MERGE_MODE
from pyams_i18n.schema import I18nTextLineField
from pyams_sequence.schema import InternalReferencesListField
from zope.schema import Choice, Int
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


#
# Views display contexts
#

VIEW_DISPLAY_CONTEXT = 'display'
VIEW_CONTENT_CONTEXT = 'content'

VIEW_CONTEXTS = (
    {'id': VIEW_DISPLAY_CONTEXT, 'title': _("Display context")},
    {'id': VIEW_CONTENT_CONTEXT, 'title': _("Content context")}
)

VIEW_CONTEXT_VOCABULARY = SimpleVocabulary([SimpleTerm(item['id'], title=item['title'])
                                            for item in VIEW_CONTEXTS])


#
# Views merge modes
#

class IViewItemsPortletSettings(IPortletSettings):
    """View items portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              required=False)

    views = InternalReferencesListField(title=_("Selected views"),
                                        description=_("Reference to the view(s) from which items are extracted; "
                                                      "you can combine several views together and specify in which "
                                                      "order they should be mixed"),
                                        content_type=WfView.content_type,
                                        required=True)

    def get_views(self):
        """Get iterator over selected views"""

    views_context = Choice(title=_("Views context"),
                           description=_("When searching for items, a view receives a \"context\" which is the object "
                                         "from which settings can be extracted; this context can be the \"display\" "
                                         "context or the \"content\" context: when the portlet is used to display the "
                                         "site root, a site manager or a site folder, both are identical; when the "
                                         "portlet is used to display a shared content, the \"content\" context is the "
                                         "displayed content, while the \"display\" context is the container (site "
                                         "root, site manager or site folder) into which content is displayed"),
                           vocabulary=VIEW_CONTEXT_VOCABULARY,
                           default=VIEW_DISPLAY_CONTEXT,
                           required=True)

    views_merge_mode = Choice(title=_("Views merge mode"),
                              description=_("If you select several views, you can select \"merge\" mode, which is "
                                            "the way used to merge items from several views"),
                              vocabulary=VIEWS_MERGERS_VOCABULARY,
                              default=CONCAT_VIEWS_MERGE_MODE,
                              required=True)

    def get_merger(self):
        """Get selected views merger utility"""

    limit = Int(title=_("Results count limit"),
                description=_("Maximum number of results that the component may extract from merged views"),
                required=False)

    start = Int(title=_("Starting from..."),
                description=_("You can skip several results if specifying an integer value here..."),
                required=False,
                default=1)

    def get_items(self):
        """Get iterator over items returned by selected views, using selected merger"""
